/*	RCT3 libOVL
	Copyright 2005 Jonathan Wilson

	This file is part of the RCT3 libOVL
	The RCT3 libOVL library is free software; you can redistribute it and/or modify it under
	the terms of the GNU General Public License as published by the Free
	Software Foundation; either version 2, or (at your option) any later
	version. See the file COPYING for more details.
*/

#include "Common.h"
#include "CommonOVL.h"
#include "StyleOVL.h"
extern FILE *f;
extern unsigned long CurrentFile; 
extern OvlFile OpenFiles[2];
extern std::queue<unsigned long *> relocations;
extern std::queue<unsigned long> fixups;
extern std::vector <libOVL_Resource *> ResourceItems;

char *stdstrings = "StyleIndex:int\0GUIIcon:txs\0GuestInjectionPoint\0LitterBin\0 Capacity 4.0\0ParkBench\0ParkEntrance\0QueueLineEntertainer\0RideEvent\0Sign\0 EnableNoEntry\0VistaPoint\0Waterable\0 Capacity 300.0\0adventure\0crazygolf\0crazygolfholes\0girder\0ornate\0pathasphalt\0pathcrazy\0pathdirt\0pathleafy\0marble\0pathtarmac\0flatqueueset1\0romanpillar\0scaffolding\0space\0spooky\0steel\0ts1\0ts2\0ts3\0ts4\0ts5\0flatwildwest\0\0MapColourBlue:flt\0MapColourRed:flt\0MapColorGreen:flt\0PathFountain\0LoudSpeaker\0PassportPost\0WaterCannon\0aquarium\0aquariumcorner\0defaultpool\0defaultpoolpath\0defaultpoolsmall\0sand\0pathunderwater\0ts1a\0ts6\0ts7\0ts8";

void clearStyleOVLData();
unsigned long* styleval;
int compare(const void *s1,const void *s2);

//Filename - style.common.ovl
LIBOVL_API unsigned long CreateStyleOvl(char *filename, unsigned long ThemeCode)
{
	char *UniqueOVLFilename = new char[strlen(filename)+1];
	strcpy(UniqueOVLFilename,filename);
	strcpy(strstr(UniqueOVLFilename,".common.ovl"),".unique.ovl");

	OpenFiles[OVL_UNIQUE].FileName = UniqueOVLFilename;
	OpenFiles[OVL_COMMON].FileName= new char[strlen(filename)+1];
	strcpy(OpenFiles[OVL_COMMON].FileName , filename);


	OpenFiles[OVL_COMMON].ReferenceCount=0;
	OpenFiles[OVL_COMMON].References=NULL;
	OpenFiles[OVL_UNIQUE].ReferenceCount=0;
	OpenFiles[OVL_UNIQUE].References=NULL;
	memset(OpenFiles[OVL_COMMON].Types,0,sizeof(OpenFiles[OVL_COMMON].Types));
	memset(OpenFiles[OVL_UNIQUE].Types,0,sizeof(OpenFiles[OVL_UNIQUE].Types));

	clearStyleOVLData();
	styleval = (unsigned long *) ThemeCode;

	return 0;

}

LIBOVL_API bool saveStyleOVL()
{
	unsigned long stringcount = 0;
	unsigned long stringtablesize = 0;
	unsigned long textstringsize = 0;
	unsigned long loadersize = 0;
	unsigned long symbolsize = 0;
	unsigned long symbolrefsize = 0;
	unsigned long flicsize = 0;
	unsigned long texturesize = 0;
	unsigned long gsisize = 0;
	unsigned long sidsize = 0;
	unsigned char *stringtable;
	unsigned char *textstrings;
	unsigned char *loaders;
	unsigned char *symbols;
	unsigned char *symrefs;
	unsigned char *flics;
	unsigned char *textures;
	unsigned char *gsis;
	unsigned char *sids;
	symbolsize += sizeof(SymbolStruct);
	symbolsize += sizeof(SymbolStruct);
	symbolsize += sizeof(SymbolStruct);
	symbolsize += sizeof(SymbolStruct);
	unsigned long i;

	for (i = 0;i < TextStrings.size();i++)
	{
		stringtablesize += strlen(TextStrings[i]->name)+5;
		textstringsize += (strlen(TextStrings[i]->text)+1)*2;
		loadersize += sizeof(LoaderStruct);
		symbolsize += sizeof(SymbolStruct);
		stringcount++;
	}
	for (i = 0;i < IconTextures.size();i++)
	{
		stringtablesize += strlen(IconTextures[i]->name)+5;
		flicsize += sizeof(FlicStruct);
		flicsize += 4;
		texturesize += sizeof(TextureStruct2);
		texturesize += sizeof(TextureStruct);
		loadersize += sizeof(LoaderStruct);
		loadersize += sizeof(LoaderStruct);
		symbolsize += sizeof(SymbolStruct);
		symbolrefsize += sizeof(SymbolRefStruct);
	}
	for (i = 0;i < Icons.size();i++)
	{
		stringtablesize += strlen(Icons[i]->name)+5;
		gsisize += sizeof(GUISkinItemPos);
		gsisize += sizeof(GUISkinItem);
		loadersize += sizeof(LoaderStruct);
		symbolsize += sizeof(SymbolStruct);
		symbolrefsize += sizeof(SymbolRefStruct);
	}
	for (unsigned long i = 0;i < SceneryItems.size();i++)
	{
		char fname[_MAX_FNAME];
		_splitpath(SceneryItems[i]->ovl3,NULL,NULL,fname,NULL);
		strchr(fname,'.')[0] = 0;
		stringtablesize += strlen(SceneryItems[i]->InternalOVLName)+5;
		stringtablesize += strlen(SceneryItems[i]->InternalOVLName)+5;
		stringtablesize += strlen(SceneryItems[i]->ovlWithoutExt) + 1;
		loadersize += sizeof(LoaderStruct);
		symbolsize += sizeof(SymbolStruct);
		symbolrefsize += sizeof(SymbolRefStruct);
		symbolrefsize += sizeof(SymbolRefStruct);
		symbolrefsize += sizeof(SymbolRefStruct);
		if (SceneryItems[i]->wallicon != 0)
		{
			symbolrefsize += sizeof(SymbolRefStruct);
			symbolrefsize += sizeof(SymbolRefStruct);
		}
		sidsize += sizeof(SceneryItem);
		sidsize += sizeof(SceneryItemData);
		sidsize += 8;
		if (SceneryItems[i]->extra != 0)
		{
			sidsize += sizeof(SceneryParams);
		}
	}
	stringtablesize += 590;
	stringtable = new unsigned char[stringtablesize];
	symbols = new unsigned char[symbolsize];
	loaders = new unsigned char[loadersize];
	symrefs = new unsigned char[symbolrefsize];
	textstrings = new unsigned char[textstringsize];
	flics = new unsigned char[flicsize];
	textures = new unsigned char[texturesize];
	gsis = new unsigned char[gsisize];
	sids = new unsigned char[sidsize];
	unsigned char *sidsx = sids;

	File Type2Files[8];
	File Type0Files[1];
	Type0Files[0].data = (unsigned long *)stringtable;
	Type0Files[0].reloffset = 0;
	Type0Files[0].size = stringtablesize;

	Type2Files[0].data = (unsigned long *)symbols;
	Type2Files[0].reloffset = Type0Files[0].size;
	Type2Files[0].size = symbolsize;
	Type2Files[1].data = (unsigned long *)loaders;
	Type2Files[1].reloffset = Type2Files[0].reloffset + Type2Files[0].size;
	Type2Files[1].size = loadersize;
	Type2Files[2].data = (unsigned long *)symrefs;
	Type2Files[2].reloffset = Type2Files[1].reloffset + Type2Files[1].size;
	Type2Files[2].size = symbolrefsize;
	Type2Files[3].data = (unsigned long *)textstrings;
	Type2Files[3].reloffset = Type2Files[2].reloffset + Type2Files[2].size;
	Type2Files[3].size = textstringsize;
	Type2Files[4].data = (unsigned long *)flics;
	Type2Files[4].reloffset = Type2Files[3].reloffset + Type2Files[3].size;
	Type2Files[4].size = flicsize;
	Type2Files[5].data = (unsigned long *)textures;
	Type2Files[5].reloffset = Type2Files[4].reloffset + Type2Files[4].size;
	Type2Files[5].size = texturesize;
	Type2Files[6].data = (unsigned long *)gsis;
	Type2Files[6].reloffset = Type2Files[5].reloffset + Type2Files[5].size;
	Type2Files[6].size = gsisize;
	Type2Files[7].data = (unsigned long *)sids;
	Type2Files[7].reloffset = Type2Files[6].reloffset + Type2Files[6].size;
	Type2Files[7].size = sidsize;
	OpenFiles[OVL_COMMON].Types[0].count = 1;
	OpenFiles[OVL_COMMON].Types[0].dataptr = Type0Files;
	OpenFiles[OVL_COMMON].Types[0].reloffset = 0;
	OpenFiles[OVL_COMMON].Types[0].size = stringtablesize;
	OpenFiles[OVL_UNIQUE].Types[0].count = 0;
	OpenFiles[OVL_UNIQUE].Types[0].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[0].reloffset = stringtablesize;
	OpenFiles[OVL_UNIQUE].Types[0].size = 0;
	OpenFiles[OVL_UNIQUE].Types[1].count = 0;
	OpenFiles[OVL_UNIQUE].Types[1].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[1].size = 0;
	OpenFiles[OVL_UNIQUE].Types[1].reloffset = stringtablesize;
	OpenFiles[OVL_UNIQUE].Types[2].reloffset = OpenFiles[OVL_COMMON].Types[1].reloffset;
	OpenFiles[OVL_UNIQUE].Types[2].dataptr = Type2Files;
	OpenFiles[OVL_UNIQUE].Types[2].count = 8;
	OpenFiles[OVL_UNIQUE].Types[2].size = symbolsize+loadersize+symbolrefsize+textstringsize+flicsize+texturesize+gsisize+sidsize;
	OpenFiles[OVL_UNIQUE].Types[3].count = 0;
	OpenFiles[OVL_UNIQUE].Types[3].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[3].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[3].size = 0;
	OpenFiles[OVL_UNIQUE].Types[4].count = 0;
	OpenFiles[OVL_UNIQUE].Types[4].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[4].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[4].size = 0;
	OpenFiles[OVL_UNIQUE].Types[5].count = 0;
	OpenFiles[OVL_UNIQUE].Types[5].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[5].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[5].size = 0;
	OpenFiles[OVL_UNIQUE].Types[6].count = 0;
	OpenFiles[OVL_UNIQUE].Types[6].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[6].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[6].size = 0;
	OpenFiles[OVL_UNIQUE].Types[7].count = 0;
	OpenFiles[OVL_UNIQUE].Types[7].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[7].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[7].size = 0;
	OpenFiles[OVL_UNIQUE].Types[8].count = 0;
	OpenFiles[OVL_UNIQUE].Types[8].dataptr = 0;
	OpenFiles[OVL_UNIQUE].Types[8].reloffset = OpenFiles[OVL_COMMON].Types[2].reloffset+OpenFiles[0].Types[2].size;
	OpenFiles[OVL_UNIQUE].Types[8].size = 0;

	char *strings = (char *)stringtable;
	wchar_t *text = (wchar_t *)textstrings;
	LoaderStruct *ldrs = (LoaderStruct *)loaders; //type 0 is text, 1 is texture, 2 is flic, 3 is GUISkinItem, 4 is SceneryItem
	SymbolStruct *syms = (SymbolStruct *)symbols;
	SymbolRefStruct *refs = (SymbolRefStruct *)symrefs;
	Tex *texstr = (Tex *)textures;
	Flic *flstr = (Flic *)flics;
	GSI *gsistr = (GSI *)gsis;
	memcpy(strings,stdstrings,590);
	char *StyleIndex = strings;
	char *GUIIcon = StyleIndex+(strlen(StyleIndex)+1);
	char *extra[14];
	char *extraparams[14];
	char *supports[35];
	char *mapblue;
	char *mapgreen;
	char *mapred;
	extra[1] = GUIIcon+(strlen(GUIIcon)+1);
	extra[2] = extra[1]+(strlen(extra[1])+1);
	extraparams[2] = extra[2]+(strlen(extra[2])+1);
	extra[3] = extraparams[2]+(strlen(extraparams[2])+1);
	extra[4] = extra[3]+(strlen(extra[3])+1);
	extra[5] = extra[4]+(strlen(extra[4])+1);
	extra[6] = extra[5]+(strlen(extra[5])+1);
	extra[7] = extra[6]+(strlen(extra[6])+1);
	extraparams[7] = extra[7]+(strlen(extra[7])+1);
	extra[8] = extraparams[7]+(strlen(extraparams[7])+1);
	extra[9] = extra[8]+(strlen(extra[8])+1);
	extraparams[9] = extra[9]+(strlen(extra[9])+1);
	supports[1] = extraparams[9]+(strlen(extraparams[9])+1);
	supports[2] = supports[1]+(strlen(supports[1])+1);
	supports[3] = supports[2]+(strlen(supports[2])+1);
	supports[4] = supports[3]+(strlen(supports[3])+1);
	supports[5] = supports[4]+(strlen(supports[4])+1);
	supports[6] = supports[5]+(strlen(supports[5])+1);
	supports[7] = supports[6]+(strlen(supports[6])+1);
	supports[8] = supports[7]+(strlen(supports[7])+1);
	supports[9] = supports[8]+(strlen(supports[8])+1);
	supports[10] = supports[9]+(strlen(supports[9])+1);
	supports[11] = supports[10]+(strlen(supports[10])+1);
	supports[12] = supports[11]+(strlen(supports[11])+1);
	supports[13] = supports[12]+(strlen(supports[12])+1);
	supports[14] = supports[13]+(strlen(supports[13])+1);
	supports[15] = supports[14]+(strlen(supports[14])+1);
	supports[16] = supports[15]+(strlen(supports[15])+1);
	supports[17] = supports[16]+(strlen(supports[16])+1);
	supports[18] = supports[17]+(strlen(supports[17])+1);
	supports[19] = supports[18]+(strlen(supports[18])+1);
	supports[20] = supports[19]+(strlen(supports[19])+1);
	supports[21] = supports[20]+(strlen(supports[20])+1);
	supports[22] = supports[21]+(strlen(supports[21])+1);
	supports[23] = supports[22]+(strlen(supports[22])+1);
	supports[0] = supports[23]+(strlen(supports[23])+1);
	extra[0] = supports[23]+(strlen(supports[23])+1);
	extraparams[0] = supports[23]+(strlen(supports[23])+1);
	extraparams[1] = supports[23]+(strlen(supports[23])+1);
	extraparams[3] = supports[23]+(strlen(supports[23])+1);
	extraparams[4] = supports[23]+(strlen(supports[23])+1);
	extraparams[5] = supports[23]+(strlen(supports[23])+1);
	extraparams[6] = supports[23]+(strlen(supports[23])+1);
	extraparams[8] = supports[23]+(strlen(supports[23])+1);
	extraparams[9] = supports[23]+(strlen(supports[23])+1);
	extraparams[10] = supports[23]+(strlen(supports[23])+1);
	extraparams[11] = supports[23]+(strlen(supports[23])+1);
	extraparams[12] = supports[23]+(strlen(supports[23])+1);
	extraparams[13] = supports[23]+(strlen(supports[23])+1);
	mapblue = extraparams[0]+1;
	mapred = mapblue+strlen(mapblue)+1;
	mapgreen = mapred+strlen(mapred)+1;
	extra[10] = mapgreen+strlen(mapgreen)+1;
	extra[11] = extra[10]+strlen(extra[10])+1;
	extra[12] = extra[11]+strlen(extra[11])+1;
	extra[13] = extra[12]+strlen(extra[12])+1;
	supports[24] = extra[13]+strlen(extra[13])+1;
	supports[25] = supports[24]+strlen(supports[24])+1;
	supports[26] = supports[25]+strlen(supports[25])+1;
	supports[27] = supports[26]+strlen(supports[26])+1;
	supports[28] = supports[27]+strlen(supports[27])+1;
	supports[29] = supports[28]+strlen(supports[28])+1;
	supports[30] = supports[29]+strlen(supports[29])+1;
	supports[31] = supports[30]+strlen(supports[30])+1;
	supports[32] = supports[31]+strlen(supports[31])+1;
	supports[33] = supports[32]+strlen(supports[32])+1;
	supports[34] = supports[33]+strlen(supports[33])+1;
	strings = supports[34]+strlen(supports[34])+1;
	syms->Symbol = StyleIndex;
	syms->data = (unsigned long *)styleval;
	syms->IsPointer = 0;
	syms++;
	syms->Symbol = mapred;
	syms->data = 0;
	syms->IsPointer = 0;
	syms++;
	syms->Symbol = mapgreen;
	syms->data = 0;
	syms->IsPointer = 0;
	syms++;
	syms->Symbol = mapblue;
	syms->data = 0;
	syms->IsPointer = 0;
	syms++;
	for (i = 0;i < TextStrings.size();i++)
	{
		char *str = strings;
		strcpy(strings,TextStrings[i]->name);
		strings += strlen(TextStrings[i]->name);
		strcpy(strings,":txt");
		strings += 5;
		mbstowcs(text,TextStrings[i]->text,strlen(TextStrings[i]->text)+1);
		ldrs->data = (unsigned long *)text;
		ldrs->LoaderType = 0;
		ldrs->Sym = syms;
		ldrs->SymbolsToResolve = 0;
		ldrs->HasExtraData = 1;
		syms->data = (unsigned long *)text;
		syms->Symbol = str;
		syms->IsPointer = 1;
		relocations.push((unsigned long *)&ldrs->data);
		relocations.push((unsigned long *)&ldrs->Sym);
		text += strlen(TextStrings[i]->text)+1;
		ldrs++;
		syms++;
	}
	for (i = 0;i < IconTextures.size();i++)
	{
		char *str = strings;
		strcpy(strings,IconTextures[i]->name);
		strings += strlen(IconTextures[i]->name);
		strcpy(strings,":tex");
		strings += 5;
		flstr->fl = &flstr->fl2;
		relocations.push((unsigned long *)&flstr->fl);
		flstr->fl2.FlicDataPtr = 0;
		flstr->fl2.unk1 = 1;
		flstr->fl2.unk2 = 1.0;
		texstr->t1.unk1 = 0x70007;
		texstr->t1.unk2 = 0x70007;
		texstr->t1.unk3 = 0x70007;
		texstr->t1.unk4 = 0x70007;
		texstr->t1.unk5 = 0x70007;
		texstr->t1.unk6 = 0x70007;
		texstr->t1.unk7 = 0x70007;
		texstr->t1.unk8 = 0x70007;
		texstr->t1.unk9 = 1;
		texstr->t1.unk10 = 8;
		texstr->t1.unk11 = 0x10;
		texstr->t1.TextureData = 0;
		texstr->t1.unk12 = 1;
		texstr->t1.ts2 = &texstr->t2;
		texstr->t1.Flic = &flstr->fl;
		texstr->t2.Texture = &texstr->t1;
		texstr->t2.Flic = &flstr->fl2;
		relocations.push((unsigned long *)&texstr->t1.ts2);
		relocations.push((unsigned long *)&texstr->t1.Flic);
		relocations.push((unsigned long *)&texstr->t2.Texture);
		relocations.push((unsigned long *)&texstr->t2.Flic);
		ldrs->data = (unsigned long *)&flstr->fl2;
		ldrs->LoaderType = 2;
		ldrs->Sym = 0;
		ldrs->SymbolsToResolve = 0;
		ldrs->HasExtraData = 1;
		relocations.push((unsigned long *)&ldrs->data);
		relocations.push((unsigned long *)&ldrs->Sym);
		ldrs++;
		ldrs->data = (unsigned long *)texstr;
		ldrs->LoaderType = 1;
		ldrs->Sym = syms;
		ldrs->SymbolsToResolve = 1;
		ldrs->HasExtraData = 0;
		syms->data = (unsigned long *)texstr;
		syms->Symbol = str;
		syms->IsPointer = 1;
		refs->ldr = ldrs;
		refs->Symbol = GUIIcon;
		refs->reference = (unsigned long *)&texstr->t1.TextureData;
		relocations.push((unsigned long *)&ldrs->data);
		relocations.push((unsigned long *)&ldrs->Sym);
		relocations.push((unsigned long *)&refs->reference);
		relocations.push((unsigned long *)&refs->ldr);
		relocations.push((unsigned long *)&refs->Symbol);
		ldrs++;
		syms++;
		refs++;
		flstr++;
		texstr++;
	}
	for (i = 0;i < Icons.size();i++)
	{
		char *str = strings;
		strcpy(strings,Icons[i]->name);
		strings += strlen(Icons[i]->name);
		strcpy(strings,":gsi");
		strings += 5;
		gsistr->g1.pos = &gsistr->g2;
		gsistr->g1.tex = 0;
		gsistr->g1.unk1 = 0;
		gsistr->g1.unk2 = 0;
		gsistr->g2.top = Icons[i]->top;
		gsistr->g2.left = Icons[i]->left;
		gsistr->g2.bottom = Icons[i]->bottom;
		gsistr->g2.right = Icons[i]->right;
		ldrs->data = (unsigned long *)gsistr;
		ldrs->LoaderType = 3;
		ldrs->Sym = syms;
		ldrs->SymbolsToResolve = 1;
		ldrs->HasExtraData = 0;
		syms->data = (unsigned long *)gsistr;
		syms->Symbol = str;
		syms->IsPointer = 1;
		refs->ldr = ldrs;
		unsigned long namel = strlen(Icons[i]->texture);
		for (char *s = (char *)stringtable;s < strings;s++)
		{
			if (memcmp(Icons[i]->texture,s,namel) == 0)
			{
				if (memcmp(s+namel,":tex",4) == 0)
				{
					refs->Symbol = s;
					break;
				}
			}
		}
		refs->reference = (unsigned long *)&gsistr->g1.tex;
		relocations.push((unsigned long *)&gsistr->g1.pos);
		relocations.push((unsigned long *)&ldrs->data);
		relocations.push((unsigned long *)&ldrs->Sym);
		relocations.push((unsigned long *)&refs->reference);
		relocations.push((unsigned long *)&refs->ldr);
		relocations.push((unsigned long *)&refs->Symbol);
		ldrs++;
		syms++;
		refs++;
		gsistr++;
	}
	for (unsigned long i = 0;i < SceneryItems.size();i++)
	{
		Scenery *sc = SceneryItems[i];
		char fname[_MAX_FNAME];
		_splitpath(sc->ovl3,NULL,NULL,fname,NULL);
		strchr(fname,'.')[0] = 0;
		char *str1 = strings;
		strcpy(strings,sc->InternalOVLName);
		strings += strlen(sc->InternalOVLName);
		strcpy(strings,":sid");
		strings += 5;
		char *str2 = strings;
		strcpy(strings,sc->InternalOVLName);
		strings += strlen(sc->InternalOVLName);
		strcpy(strings,":svd");
		strings += 5;
		char *str3 = strings;
		strcpy(strings,sc->ovlWithoutExt);
		strings += strlen(sc->ovlWithoutExt)+1;
		SceneryItem *s = (SceneryItem *)sids;
		sids += sizeof(SceneryItem);
		SceneryItemData *s2 = (SceneryItemData *)sids;
		sids += sizeof(SceneryItemData);
		long *s3 = (long *)sids;
		sids += 4;
		SceneryItemVisual **svd = (SceneryItemVisual **)sids;
		sids += 4;
		SceneryParams *s4 = 0;
		if (sc->extra != 0)
		{
			s4 = (SceneryParams *)sids;
			sids += sizeof(SceneryParams);
		}
		s->unk1 = 0;
		s->unkf1 = sc->flag1;
		s->unkf2 = sc->flag2;
		s->unkf3 = sc->flag3;
		s->unkf4 = sc->flag4;
		s->unkf5 = sc->flag5;
		s->unkf6 = sc->flag6;
		s->unkf7 = sc->flag7;
		s->unkf8 = sc->flag8;
		s->unkf9 = sc->flag9;
		s->unkf10 = sc->flag10;
		s->unkf11 = sc->flag11;
		s->unkf12 = sc->flag12;
		s->unkf13 = sc->flag13;
		s->unkf14 = sc->flag14;
		s->unkf15 = sc->flag15;
		s->unkf16 = sc->flag16;
		s->unkf17 = sc->flag17;
		s->unkf18 = sc->flag18;
		s->unkf19 = sc->flag19;
		s->unkf20 = sc->flag20;
		s->unkf21 = sc->flag21;
		s->unkf22 = sc->flag22;
		s->unkf23 = sc->flag23;
		s->unkf24 = sc->flag24;
		s->unkf25 = sc->flag25;
		s->unkf26 = sc->flag26;
		s->unkf27 = sc->flag27;
		s->unkf28 = sc->flag28;
		s->unkf29 = sc->flag29;
		s->unkf30 = sc->flag30;
		s->unkf31 = sc->flag31;
		s->unkf32 = sc->flag32;
		s->size = sc->sizeflag;
		s->unk4 = sc->unk4;
		s->xsquares = sc->sizex;
		s->ysquares = sc->sizey;
		s->data = s2;
		relocations.push((unsigned long *)&s->data);
		s->xpos = sc->position.x;
		s->ypos = sc->position.y;
		s->zpos = sc->position.z;
		s->xsize = sc->size.x;
		s->ysize = sc->size.y;
		s->zsize = sc->size.z;
		s->unk13 = 0;
		s->unk14 = 0;
		s->cost = sc->cost;
		s->refund = sc->refund;
		s->unk17 = 0xFFFFFFFF;
		if (sc->unk17 == 0)
		{
			s->unk17 = 0;
		}
		if (sc->unk17 == 1)
		{
			s->unk17 = 4;
		}
		if (sc->unk17 == 2)
		{
			s->unk17 = 5;
		}
		s->type = sc->type;
		s->supports = supports[sc->supportstype];
		relocations.push((unsigned long *)&s->supports);
		s->svdcount = 1;
		s->svd = svd;
		relocations.push((unsigned long *)&s->svd);
		s->gsi = 0;
		s->wallsetgsi = 0;
		s->wallsetname = 0;
		s->OvlName = str3;
		relocations.push((unsigned long *)&s->OvlName);
		if (s4 != 0)
		{
			s->params = s4;
			relocations.push((unsigned long *)&s->params);
			s->count = 1;
			s4->type = extra[sc->extra];
			s4->params = extraparams[sc->extra];
			relocations.push((unsigned long *)&s4->type);
			relocations.push((unsigned long *)&s4->params);
		}
		else
		{
			s->params = 0;
			s->count = 0;
		}
		s->SoundsCount = 0;
		s->Sounds = 0;
		s->Name = 0;
		s->unk27 = 0;
		s->unk28 = 0;
		s->unk29 = 0;
		s->unk30 = 0;
		s->unk31 = sc->unk31;
		s->unk32 = sc->unk32;
		s->unk33 = sc->unk33;
		s->unk34 = 0;
		s->unk35 = 0;
		s->unk36 = 0;
		s->unk37 = 0;
		s->unk38 = 0;
		s->unk39 = 0;
		s->unk40 = 0;
		s->unk41 = 0;
		s->unk42 = 0;
		s->unk43 = 0;
		s->unk44 = 0;
		s->unk45 = 0;
		s->unk46 = 0;
		s->unk47 = 0;
		s->unk48 = 0;
		s2->unkf33 = sc->flag33;
		s2->unkf34 = sc->flag34;
		s2->unkf35 = sc->flag35;
		s2->unkf36 = sc->flag36;
		s2->unkf37 = sc->flag37;
		s2->unkf38 = sc->flag38;
		s2->unkf39 = sc->flag39;
		s2->unkf40 = sc->flag40;
		s2->unkf41 = sc->flag41;
		s2->unkf42 = sc->flag42;
		s2->unkf43 = sc->flag43;
		s2->unkf44 = sc->flag44;
		s2->unkf45 = sc->flag45;
		s2->unkf46 = sc->flag46;
		s2->unkf47 = sc->flag47;
		s2->unkf48 = sc->flag48;
		s2->unkf49 = sc->flag49;
		s2->unkf50 = sc->flag50;
		s2->unkf51 = sc->flag51;
		s2->unkf52 = sc->flag52;
		s2->unkf53 = sc->flag53;
		s2->unkf54 = sc->flag54;
		s2->unkf55 = sc->flag55;
		s2->unkf56 = sc->flag56;
		s2->unkf57 = sc->flag57;
		s2->unkf58 = sc->flag58;
		s2->unkf59 = sc->flag59;
		s2->unkf60 = sc->flag60;
		s2->unkf61 = sc->flag61;
		s2->unkf62 = sc->flag62;
		s2->unkf63 = sc->flag63;
		s2->unkf64 = sc->flag64;
		s2->unk2 = sc->dunk2;
		s2->unk3 = sc->dunk3;
		if (sc->dunk4 != 0xFFFFFFFF)
		{
			s2->unk4 = s3;
		}
		else
		{
			s2->unk4 = 0;
		}
		relocations.push((unsigned long *)&s2->unk4);
		s2->unk5 = sc->dunk5;
		*s3 = sc->dunk4;
		*svd = 0;
		ldrs->data = (unsigned long *)s;
		ldrs->LoaderType = 4;
		ldrs->Sym = syms;
		ldrs->HasExtraData = 0;
		syms->data = (unsigned long *)s;
		syms->Symbol = str1;
		syms->IsPointer = 1;
		relocations.push((unsigned long *)&ldrs->data);
		relocations.push((unsigned long *)&ldrs->Sym);
		refs->ldr = ldrs;
		unsigned long namel = strlen(sc->icon);
		for (char *st = (char *)stringtable;st < strings;st++)
		{
			if (memcmp(sc->icon,st,namel) == 0)
			{
				if (memcmp(st+namel,":gsi",4) == 0)
				{
					refs->Symbol = st;
					break;
				}
			}
		}
		refs->reference = (unsigned long *)&s->gsi;
		relocations.push((unsigned long *)&refs->reference);
		relocations.push((unsigned long *)&refs->ldr);
		relocations.push((unsigned long *)&refs->Symbol);
		refs++;
		refs->ldr = ldrs;
		ldrs->SymbolsToResolve = 5;
		if (sc->wallicon != 0)
		{
			namel = strlen(sc->wallicon);
			for (char *st = (char *)stringtable;st < strings;st++)
			{
				if (memcmp(sc->wallicon,st,namel) == 0)
				{
					if (memcmp(st+namel,":gsi",4) == 0)
					{
						refs->Symbol = st;
						break;
					}
				}
			}
			refs->reference = (unsigned long *)&s->wallsetgsi;
			relocations.push((unsigned long *)&refs->reference);
			relocations.push((unsigned long *)&refs->ldr);
			relocations.push((unsigned long *)&refs->Symbol);
			refs++;
		}
		else
		{
			ldrs->SymbolsToResolve--;
			refs->Symbol = 0;
		}
		refs->ldr = ldrs;
		namel = strlen(sc->name);
		for (char *st = (char *)stringtable;st < strings;st++)
		{
			if (memcmp(sc->name,st,namel) == 0)
			{
				if (memcmp(st+namel,":txt",4) == 0)
				{
					refs->Symbol = st;
					break;
				}
			}
		}
		refs->reference = (unsigned long *)&s->Name;
		relocations.push((unsigned long *)&refs->reference);
		relocations.push((unsigned long *)&refs->ldr);
		relocations.push((unsigned long *)&refs->Symbol);
		refs++;
		if (sc->wallname != 0)
		{
			refs->ldr = ldrs;
			namel = strlen(sc->wallname);
			for (char *st = (char *)stringtable;st < strings;st++)
			{
				if (memcmp(sc->wallname,st,namel) == 0)
				{
					if (memcmp(st+namel,":txt",4) == 0)
					{
						refs->Symbol = st;
						break;
					}
				}
			}
			refs->reference = (unsigned long *)&s->wallsetname;
			relocations.push((unsigned long *)&refs->reference);
			relocations.push((unsigned long *)&refs->ldr);
			relocations.push((unsigned long *)&refs->Symbol);
			refs++;
		}
		else
		{
			ldrs->SymbolsToResolve--;
			refs->Symbol = 0;
		}
		refs->ldr = ldrs;
		refs->Symbol = str2;
		refs->reference = (unsigned long *)svd;
		relocations.push((unsigned long *)&refs->reference);
		relocations.push((unsigned long *)&refs->ldr);
		relocations.push((unsigned long *)&refs->Symbol);
		refs++;
		ldrs++;
		syms++;
	}
	SymbolStruct *syms2 = (SymbolStruct *)symbols;
	qsort(syms2,symbolsize/sizeof(SymbolStruct),sizeof(SymbolStruct),compare);
	for (i = 0;i < symbolsize/sizeof(SymbolStruct);i++)
	{
		relocations.push((unsigned long *)&(syms2[i].Symbol));
		if ((syms2[i].Symbol != mapblue) && (syms2[i].Symbol != mapgreen) && (syms2[i].Symbol != mapred) && (syms2[i].Symbol != StyleIndex))
		{
			relocations.push((unsigned long *)&(syms2[i].data));
		}
	}
	LoaderStruct *ldrs2 = (LoaderStruct *)loaders;
	for (i = 0;i < loadersize/sizeof(LoaderStruct);i++)
	{
		for (unsigned long j = 0;j < symbolsize/sizeof(SymbolStruct);j++)
		{
			if (ldrs2[i].data == syms2[j].data)
			{
				ldrs2[i].Sym = &syms2[j];
			}
		}
	}
	OvlHeader h;
	h.magic = 0x4B524746;
	h.References = 0;
	h.reserved = 0;
	h.version = 1;
	OvlHeader2 h2;
	h2.FileTypeCount = 5;
	h2.unk = 0;
	unsigned short len1 = 4;
	char *str1 = "FGDK";
	unsigned short len2 = 4;
	char *str2 = "Text";
	unsigned long type = 2;
	unsigned short len3 = 3;
	char *str3 = "txt";
	FILE *f = fopen(OpenFiles[OVL_UNIQUE].FileName,"wb");
	fwrite(&h,sizeof(h),1,f);
	fwrite(&h2,sizeof(h2),1,f);
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 7;
	str2 = "Texture";
	type = 2;
	len3 = 3;
	str3 = "tex";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 4;
	str2 = "Flic";
	type = 2;
	len3 = 4;
	str3 = "flic";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 0xB;
	str2 = "GUISkinItem";
	type = 1;
	len3 = 3;
	str3 = "gsi";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "SceneryItem";
	type = 1;
	len3 = 3;
	str3 = "sid";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	for (i = 0;i < 9;i++)
	{
		fwrite(&OpenFiles[OVL_UNIQUE].Types[i].count,sizeof(OpenFiles[OVL_UNIQUE].Types[i].count),1,f);
	}

	CurrentFile =OVL_UNIQUE ;

	while (relocations.empty() == false)
	{
		unsigned long *relocation = relocations.front();
		relocations.pop();
		unsigned long *reloc = *(unsigned long **)relocation;
		if (reloc != 0)
		{
			unsigned long fixup = DoRelocationForSaving(reloc);
			if (fixup == 0xFFFFFFFF)
			{
				fixup = 0;
			}
			*relocation = fixup;
			unsigned long fixup2 = DoRelocationForSaving(relocation);
			fixups.push(fixup2);
		}
	}
	for (i = 0;i < 9;i++)
	{
		for (unsigned long j = 0;j < OpenFiles[OVL_UNIQUE].Types[i].count;j++)
		{
			fwrite(&OpenFiles[OVL_UNIQUE].Types[i].dataptr[j].size,sizeof(OpenFiles[OVL_UNIQUE].Types[i].dataptr[j].size),1,f);
			fwrite(OpenFiles[OVL_UNIQUE].Types[i].dataptr[j].data,OpenFiles[OVL_UNIQUE].Types[i].dataptr[j].size,1,f);
		}
	}
	unsigned long size = fixups.size();
	fwrite(&size,sizeof(size),1,f);
	while (fixups.empty() == false)
	{
		unsigned long fixup = fixups.front();
		fixups.pop();
		fwrite(&fixup,sizeof(fixup),1,f);
	}
	int t1 = 4;
	int t2 = 1;
	for (i = 0;i < TextStrings.size();i++)
	{
		fwrite(&t1,4,1,f);
		fwrite(&t2,4,1,f);
	}
	for (i = 0;i < IconTextures.size();i++)
	{
		unsigned long imgsize = IconTextures[i]->fmh.Blocks * IconTextures[i]->fmh.Pitch;
		unsigned long size = sizeof(FlicHeader) + sizeof(FlicMipHeader) + imgsize + sizeof(FlicMipHeader);
		fwrite(&size,4,1,f);
		fwrite(&IconTextures[i]->fh,sizeof(FlicHeader),1,f);
		fwrite(&IconTextures[i]->fmh,sizeof(FlicMipHeader),1,f);
		fwrite(IconTextures[i]->data,imgsize,1,f);
		IconTextures[i]->fmh.Blocks = 0;
		IconTextures[i]->fmh.MHeight = 0;
		IconTextures[i]->fmh.MWidth = 0;
		IconTextures[i]->fmh.Pitch = 0;
		fwrite(&IconTextures[i]->fmh,sizeof(FlicMipHeader),1,f);
	}
	fclose(f);
	h.magic = 0x4B524746;
	h.References = 0;
	h.reserved = 0;
	h.version = 1;
	h2.FileTypeCount = 5;
	h2.unk = 0;
	len1 = 4;
	str1 = "FGDK";
	len2 = 4;
	str2 = "Text";
	type = 2;
	len3 = 3;
	str3 = "txt";
	strcpy(strstr(OpenFiles[OVL_UNIQUE].FileName,".unique.ovl"),".common.ovl");
	strcpy(OpenFiles[0].FileName,OpenFiles[OVL_UNIQUE].FileName);
	f = fopen(OpenFiles[0].FileName,"wb");
	fwrite(&h,sizeof(h),1,f);
	fwrite(&h2,sizeof(h2),1,f);
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 7;
	str2 = "Texture";
	type = 2;
	len3 = 3;
	str3 = "tex";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 4;
	str2 = "Flic";
	type = 2;
	len3 = 4;
	str3 = "flic";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "FGDK";
	len2 = 0xB;
	str2 = "GUISkinItem";
	type = 1;
	len3 = 3;
	str3 = "gsi";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	len1 = 4;
	str1 = "RCT3";
	len2 = 0xB;
	str2 = "SceneryItem";
	type = 1;
	len3 = 3;
	str3 = "sid";
	fwrite(&len1,sizeof(len1),1,f);
	fwrite(str1,len1,1,f);
	fwrite(&len2,sizeof(len2),1,f);
	fwrite(str2,len2,1,f);
	fwrite(&type,sizeof(type),1,f);
	fwrite(&len3,sizeof(len3),1,f);
	fwrite(str3,len3,1,f);
	unsigned long tmp = 1;
	fwrite(&tmp,4,1,f);
	tmp = 0;
	for (i = 0;i < 8;i++)
	{
		fwrite(&tmp,4,1,f);
	}
	fwrite(&Type0Files[0].size,4,1,f);
	fwrite(Type0Files[0].data,Type0Files[0].size,1,f);
	fwrite(&tmp,4,1,f);
	fclose(f);
	//delete data;
	delete sidsx;
	delete gsis;
	delete textures;
	delete flics;
	delete textstrings;
	delete symrefs;
	delete loaders;
	delete symbols;
	delete stringtable;
	
	return 0;
}


LIBOVL_API void clearStyleOVLData()
{
	DeleteStyleOVLInfo_Icons();
	DeleteStyleOVLInfo_IconTextures();
	DeleteStyleOVLInfo_SceneryItems();
	DeleteStyleOVLInfo_TextStrings();
	DeleteInfo_Resources();
	DeleteInfo_FileReferences(OVL_COMMON);
	DeleteInfo_FileReferences(OVL_UNIQUE);

}

LIBOVL_API void AddStyleOVLInfo_SceneryItem(Scenery *scn)
{
	long curItem=-1;

	Scenery *t =new Scenery;
	(*t)=(*scn);
	SceneryItems.push_back(t);
	curItem = SceneryItems.size()-1;
	
	Scenery *s = SceneryItems[curItem];

	s->name = new char[strlen(scn->name)+1];
	strcpy(s->name,scn->name);
	s->ovl2 = new char[strlen(scn->ovl2)+1];
	strcpy(s->ovl2,scn->ovl2);
	s->ovl3 = new char[strlen(scn->ovl3)+1];
	strcpy(s->ovl3,scn->ovl3);
	s->ovlWithoutExt = new char[strlen(scn->ovlWithoutExt)+1];
	strcpy(s->ovlWithoutExt,scn->ovlWithoutExt);
	s->InternalOVLName = new char[strlen(scn->InternalOVLName)+1];
	strcpy(s->InternalOVLName,scn->InternalOVLName);
	s->location = new char[strlen(scn->location)+1];
	strcpy(s->location,scn->location);
	s->icon = new char[strlen(scn->icon)+1];
	strcpy(s->icon,scn->icon);

	if (scn->wallicon != 0)
	{
		s->wallicon = new char[strlen(scn->wallicon)+1];
		strcpy(s->wallicon,scn->wallicon);
		s->wallname = new char[strlen(scn->wallname)+1];
		strcpy(s->wallname,scn->wallname);
	}

}

LIBOVL_API void DeleteStyleOVLInfo_SceneryItems()
{
	for (unsigned long i = 0;i < SceneryItems.size();i++)
	{
		delete SceneryItems[i]->name;
		delete SceneryItems[i]->ovl2;
		delete SceneryItems[i]->ovl3;
		delete SceneryItems[i]->location;
		delete SceneryItems[i]->icon;
		if (SceneryItems[i]->wallicon != 0)
		{
			delete SceneryItems[i]->wallicon;
			delete SceneryItems[i]->wallname;
		}
		delete SceneryItems[i];
	}
	
	SceneryItems.clear();
}

LIBOVL_API void AddStyleOVLInfo_IconTexture(IconTexture *ict)
{
	long curItem=-1;

	IconTexture *t = new IconTexture;
	(*t)=(*ict);
	IconTextures.push_back(t);
	curItem = IconTextures.size()-1;
	IconTexture *s = IconTextures[curItem];

	s->name = new char[strlen(ict->name)+1];
	strcpy(s->name,ict->name);
	s->filename = new char[strlen(ict->filename)+1];
	strcpy(s->filename,ict->filename);
	s->data = new unsigned char[ict->fmh.Pitch * ict->fmh.Blocks];
	memcpy(s->data,ict->data,ict->fmh.Pitch * ict->fmh.Blocks);

}

LIBOVL_API void DeleteStyleOVLInfo_IconTextures()
{
	for (unsigned long i = 0;i < IconTextures.size();i++)
	{
		delete IconTextures[i]->filename;
		delete IconTextures[i]->name;
		delete IconTextures[i]->data;
		delete IconTextures[i];
	}
	
	IconTextures.clear();
}

LIBOVL_API void AddStyleOVLInfo_Icon(Icon *ico)
{
	long curItem=-1;

	Icon *t = new Icon;
	(*t)=(*ico);
	Icons.push_back(t);
	curItem = Icons.size()-1;
	Icon *s = Icons[curItem];

	s->name = new char[strlen(ico->name)+1];
	strcpy(s->name,ico->name);
	s->texture = new char[strlen(ico->texture)+1];
	strcpy(s->texture,ico->texture);

}

LIBOVL_API void DeleteStyleOVLInfo_Icons()
{
	for	(unsigned long i = 0;i < Icons.size();i++)
	{
		delete Icons[i]->name;
		delete Icons[i]->texture;
		delete Icons[i];
	}
	
	Icons.clear();
}

LIBOVL_API void AddStyleOVLInfo_TextString(Text *txt)
{
	long curItem=-1;
	Text *t = new Text;
	TextStrings.push_back(t);
	curItem = TextStrings.size()-1;
	Text *s = TextStrings[curItem];

	s->name = new char[strlen(txt->name)+1];
	strcpy(s->name,txt->name);
	s->text = new char[strlen(txt->text)+1];
	strcpy(s->text,txt->text);

}

LIBOVL_API void DeleteStyleOVLInfo_TextStrings()
{
	for (unsigned long i = 0;i < TextStrings.size();i++)
	{
		delete TextStrings[i]->name;
		delete TextStrings[i]->text;
		delete TextStrings[i];
	}
	
	TextStrings.clear();
}

int compare(const void *s1,const void *s2)
{
	return stricmp(((SymbolStruct *)s1)->Symbol,((SymbolStruct *)s2)->Symbol);
}
